// SN74HC595N

// the following lines is supposed(optional) to replace a single line in sketch, the #include"Esp8266MyWiFiUnified.h".
// For the following macros copied to main for easy modify. So, copy these to there and uncomment it could simply done.
#ifndef _ESP8266_MY_WIFI_UNIFIED_DISABLEH_
#define _ESP8266_MY_WIFI_UNIFIED_DISABLEH_
#include <ESP8266WiFi.h>

////////////////// paste the macros here (which is between IoT BEGIN/END .h) //////////////////
// ************************** IoT BEGIN .h ******************************
// ========================== Options and Defs/modify if need ==============================
// -------------------------- for AP or STA ------------------------------
// 1 for ap, 0 for sta.
#define AP_MODE 1
#define STA_MODE (!AP_MODE)

// -------------------------- for ap/sta credential ------------------------------ comment it and put it in .cpp
#if 1
const char *const WIFI_CRED[][2]={
    {"ESP_TESTAP", "87654321"},
    {"Office", "123456"},
    {"Xperia X Performance_bc10", "e848acbcfd98"},
};
#endif // if 1

// -------------------------- for part of hostname ------------------------------
#define LSERVICE_1 "sn74hc595n"
#define LPSERVICE_1 80

// -------------------------- for default connection names ------------------------------
// i.e., http://hostname.local/
#define HOSTNAME_BASE "esp8266-iot" // for local used.
#define HOSTNAME ((String(HOSTNAME_BASE)+"-"+LSERVICE_1).c_str()) // for local used.
#define REMOTE_HOSTNAME "esp8266-iot-internet" // for remote use.

#define UPDFILE "a.bin" // the remote webpage for updating to local is http://REMOTE_HOSTNAME.local/upd

// -------------------------- other settings ------------------------------
#define HTTPC_GET_DELAY delay(25) // do the delay after an HTTP GET.
#define HTTPC_GET_PROC_DELAY delay(5) // do some delay during an HTTP GET course.
#define HTTPC_CONN_TIMEOUT 3000 // the timeout during a GET, 3000ms.

// all the TMO values is restricted to no more than 35 minutes; see the Timeout() for detail. however,
// cascading could overcome.
#define TMO_IOT_LOOP_PERIOD 75 // IoT polling at each 75ms
#define TMO_AP_C_CONN_PERIOD 30000 // each 30 seconds to show connections by AP
#define TMO_STA_CONN_HEALTH_PERIOD 20000 // each 20 seconds to check the connection health by STA(while ping is alive)
#define TMO_WEB_UPDATE_PERIOD (1000*60*30) // each 30 minutes to find internet update; if cascade twice(no, use count) will become 15hr
#define TMO_MDNS_Q_PERIOD 120000 // each 60 seconds to query services
#define TMO_PING_PERIOD 6000 // each 3 seconds to ping AP when ping failed

#define STA_RECOVER_ALIVE 0 // set 0 if STA connection broken and do not reconnect. 1 to reconnect.

// [mDns] use either of one
#define USE_MDNS_STANDALONE // no auto hostname
//#define USE_MDNS_FULL // auto hostname and query

// [STAtion] use either of one
//#define USE_STA_NATIVE // single AP
#define USE_STA_WIFI_MULTI // multiple APs(o)

// [HTTPClient] use either of one
//#define USE_HTTPC_HTTP_CLIENT
//#define USE_HTTPC_WIFI_CLIENT
#define USE_HTTPC_WIFIHTTP_HYBRID // (o2)
//#define USE_HTTPC_WIFIHTTP_HYBRID_NONBLOCKING // (o1)

// -------------------------- for mdns ------------------------------
typedef struct smDnsRecord{
    const char *service;
    const unsigned port;
} smDnsRecord;

// -------------------------- for mdns services ------------------------------ comment it and put it in .cpp
#if 1
#if defined(USE_MDNS_FULL)
const smDnsRecord mdns_ss[]={ // the services this device provides; note, at least the LSERVICE_1 has already provided.
    {"display", 80},
    {"iot", 80},
    {"https", 443},
    {"test", 81},
    {"motor", 80},
};
#endif // USE_MDNS_FULL

const smDnsRecord mdns_sskn[]={ // the known services this device needs from other devices.
    {"display", 80},
    {"iot", 80},
    {"https", 443},
    {"test", 81},
    {"motor", 80},
};
#endif // if 1


// ************************** IoT END .h ******************************
////////////////// paste the macros here (which is between IoT BEGIN/END .h) //////////////////

#include"Esp8266MyWiFiUnified.h"
#endif // _ESP8266_MY_WIFI_UNIFIED_DISABLEH_


#include"Esp8266MultiPwms.h"
#include"Esp8266Vars2File.h"


#define VERSION_202105100001 "[[202105100001][AP/STA][ESP_TESTAP:87654321][esp8266-iot-sn74hc595n:80][esp8266-iot-internet][upd:a.bin]"\
            "[MDNS:FIXED][unknown1][115200:160:26:1M:128KFS:512KSK:80M]]" // so the version number is the 3rd byte, length 12.




// ------------------------------------ for periodically extended gpio polling
#define EXIO_INT_GPIO           12
#define EXIO_TRG_GPIO           13
#define EXIO_PERIOD_PRE         900000 // 900ms. less than 900ms will wdt-reset when sta mode in connecting.
#define EXIO_PERIOD             30000 // 30ms. less than 30ms will wdt-reset when ap mode.

// ------------------------ shift register
#define SFREG_SRCLK_GPIO        16
#define SFREG_RCLK_GPIO         3
#define SFREG_SER_GPIO          5

#define VOLTAGE_REAL            3.304f
#define VOLTAGE_TRAN            0.931f
#define VOFFSET                 3
#define BVOLTAGE(x)             (((x)>VOFFSET? (x)-VOFFSET: 0)*(1.0f/VOLTAGE_TRAN*VOLTAGE_REAL/1024.f))


unsigned stop_the_timer=0; // must stop the timer before webupdate




IRAM_ATTR unsigned exio_states[8]={0}; // note: high bit first
IRAM_ATTR unsigned exio_gate=0;
IRAM_ATTR unsigned exio_change=0;

cHwTimer timer_extend_gpio_manip; // be aware that hwtimer1 must bring up first.
IRAM_ATTR unsigned timer_trigger=0;
IRAM_ATTR void isr_exio_manip_timer(){digitalWrite(EXIO_TRG_GPIO, timer_trigger^=1);}
unsigned the_exio_period=0;

void ExioSetup(){
    pinMode(EXIO_TRG_GPIO, OUTPUT); // this gpio is to trigger intr gpio
    digitalWrite(EXIO_TRG_GPIO, LOW);
    attachInterrupt(digitalPinToInterrupt(EXIO_INT_GPIO), isr_exio_manip, CHANGE);

#if AP_MODE
    timer_extend_gpio_manip.setTimer(EXIO_PERIOD, isr_exio_manip_timer);
    the_exio_period=EXIO_PERIOD;
#else
    timer_extend_gpio_manip.setTimer(EXIO_PERIOD_PRE, isr_exio_manip_timer);
    the_exio_period=EXIO_PERIOD_PRE;
#endif
}

IRAM_ATTR void isr_exio_manip(){
    if (exio_gate || !exio_change) return;
    exio_change=0;

    for (int i=0; i<8; i++){
        digitalWrite(SFREG_SER_GPIO, exio_states[i]? HIGH: LOW);
        delayMicroseconds(1);
        digitalWrite(SFREG_SRCLK_GPIO, HIGH);
        delayMicroseconds(1);
        digitalWrite(SFREG_SRCLK_GPIO, LOW);
        delayMicroseconds(1);
    }
    digitalWrite(SFREG_SER_GPIO, LOW);
    digitalWrite(SFREG_RCLK_GPIO, HIGH);
    delayMicroseconds(1);
    digitalWrite(SFREG_RCLK_GPIO, LOW);
    delayMicroseconds(1);
}


// ------------------------------------ http
String web_link;
void my_http_request(){ // called only if http_request_ip assigned.
    static unsigned tmo=micros();
    if (Timeout(433, tmo)){ // each 0.433 seconds to send http request.
        HTTPC_http_client_connect(web_link.c_str());
    }
}

void my_setup_http_server(){

    uri_list+=\
        "/version\r\n"\
        "/stop (stop the timer)\r\n";

    server_p->on("/version", []() { // show version
        server_p->send(200, "text/plain", VERSION_202105100001);
    });

    server_p->on("/stop", []() { // stop the timer
        extern unsigned stop_the_timer;
        stop_the_timer=1;
        server_p->send(200, "text/plain", "Done");
    });

}


void init_params(){
    cDataBase s("/settings");
    s.Open();
    unsigned BootTimes=0;

    if (s.ReadWords(BootTimes, "BootTimes", 0)){
        printf("load params success: %s %u\r\n", "BootTimes", BootTimes);
    }

    s.Close();
    s.ReplaceWordsEx("BootTimes", 0, ++BootTimes);
    if (!(BootTimes%5)) s.Renew();
}


void setup(){
    Serial.begin(115200);
    while (!Serial) delay(5);

    init_params();

    randomSeed(micros()%ESP.getChipId());

    setup_IoT();

    pinMode(SFREG_SRCLK_GPIO, OUTPUT);
    digitalWrite(SFREG_SRCLK_GPIO, LOW);

    pinMode(SFREG_RCLK_GPIO, OUTPUT);
    digitalWrite(SFREG_RCLK_GPIO, LOW);

    pinMode(SFREG_SER_GPIO, OUTPUT);
    digitalWrite(SFREG_SER_GPIO, LOW);

    ExioSetup();

    for (int i=0; i<8; i++) exio_states[i]=1;
}


void loop(){

    if (stop_the_timer){
        if (stop_the_timer==1){
            stop_the_timer=2;
            timer_extend_gpio_manip.Stop();
        }
    }

    static unsigned tmo1=micros();
    if (Timeout(1500, tmo1)){
        printf("[%d, %f]\r\n", analogRead(A0), BVOLTAGE(analogRead(A0)));
    }

    static unsigned tmo11=micros();
    if (Timeout(131, tmo11)){
        exio_gate=1;

        // ------------ led control -----------
        static int t=random(8), v=1;
        exio_states[t]=1; // turn off
        if (t<=0) v=1;
        else if (t>=7) v=-1;
        exio_states[t+=v]=0; // turn on
        // -----------------------

        exio_change=1;
        exio_gate=0;
    }

    static unsigned tmo111=micros();
    if (Timeout(71, tmo111)){
        exio_gate=1;

        // ------------ led control -----------
        static int tt=random(8), vv=-1;
        exio_states[tt]=1; // turn off
        if (tt<=0) vv=1;
        else if (tt>=7) vv=-1;
        exio_states[tt+=vv]=0; // turn on
        // -----------------------

        exio_change=1;
        exio_gate=0;
    }

    static unsigned x=1;
    if (x && *remote_ap_ip){

#if STA_MODE
    timer_draw_manip.setTimer(EXIO_PERIOD, isr_exio_manip_timer); // reconfig timer
    the_exio_period=EXIO_PERIOD;
#endif

        if (!*http_request_ip) strcpy(http_request_ip, remote_ap_ip);
        web_link=String("http://")+http_request_ip+"/s";
        x=0;
    }
    loop_IoT();
}
